@@ -41,15 +41,16 @@ const PLAYBACK_TYPES: [&rspotify::model::AdditionalType; 2] = [
 
 /// The application's Spotify client
 #[derive(Clone)]
-pub struct Client {
+pub struct AppClient {
     http: reqwest::Client,
     spotify: Arc<spotify::Spotify>,
     auth_config: AuthConfig,
+    user_client: Option<rspotify::AuthCodePkceSpotify>,
     #[cfg(feature = "streaming")]
-    stream_conn: Arc<Mutex<Option<librespot_connect::spirc::Spirc>>>,
+    stream_conn: Arc<Mutex<Option<librespot_connect::Spirc>>>,
 }
 
-impl Deref for Client {
+impl Deref for AppClient {
     type Target = spotify::Spotify;
     fn deref(&self) -> &Self::Target {
         self.spotify.as_ref()
@@ -60,17 +61,49 @@ fn market_query() -> Query<'static> {
     Query::from([("market", "from_token")])
 }
 
-impl Client {
+impl AppClient {
     /// Construct a new client
-    pub fn new(auth_config: AuthConfig) -> Self {
-        Self {
+    pub async fn new() -> Result<Self> {
+        let configs = config::get_config();
+        let auth_config = AuthConfig::new(configs)?;
+
+        // Construct user-provided client.
+        // This custom client is needed for Spotify Connect integration because the Spotify client (`AppConfig::spotify`),
+        // which `spotify-player` uses to retrieve Spotify data, doesn't have access to user available devices
+        let mut user_client = configs.app_config.get_user_client_id()?.clone().map(|id| {
+            let creds = rspotify::Credentials { id, secret: None };
+            let oauth = rspotify::OAuth {
+                scopes: rspotify::scopes!("user-read-playback-state"),
+                redirect_uri: configs.app_config.login_redirect_uri.clone(),
+                ..Default::default()
+            };
+            let config = rspotify::Config {
+                token_cached: true,
+                cache_path: configs.cache_folder.join("user_client_token.json"),
+                ..Default::default()
+            };
+            rspotify::AuthCodePkceSpotify::with_config(creds, oauth, config)
+        });
+
+        if let Some(client) = &mut user_client {
+            let url = client
+                .get_authorize_url(None)
+                .context("get authorize URL for user-provided client")?;
+            client
+                .prompt_for_token(&url)
+                .await
+                .context("get token for user-provided client")?;
+        }
+
+        Ok(Self {
             spotify: Arc::new(spotify::Spotify::new()),
             http: reqwest::Client::new(),
             auth_config,
+            user_client,
 
             #[cfg(feature = "streaming")]
             stream_conn: Arc::new(Mutex::new(None)),
-        }
+        })
     }
 
     /// Initialize the application's playback upon creating a new session or during startup
@@ -619,16 +652,15 @@ impl Client {
     }
 
     /// Get user available devices
-    // This is a custom API to replace `rspotify::device` API to support Spotify Connect feature
     pub async fn available_devices(&self) -> Result<Vec<rspotify::model::Device>> {
-        Ok(self
-            .http_get::<rspotify::model::DevicePayload>(
-                &format!("{SPOTIFY_API_ENDPOINT}/me/player/devices"),
-                &Query::new(),
-                true,
-            )
-            .await?
-            .devices)
+        match &self.user_client {
+            None => {
+                tracing::warn!("User-provided client integration is not enabled, no device found.");
+                tracing::warn!("Please make sure you setup Spotify Connect as described in https://github.com/aome510/spotify-player#spotify-connect.");
+                Ok(vec![])
+            }
+            Some(client) => Ok(client.device().await?),
+        }
     }
 
     pub fn update_playback(&self, state: &SharedState) {
@@ -674,13 +706,7 @@ impl Client {
     /// Find an available device. If found, return the device's ID.
     async fn find_available_device(&self) -> Result<Option<String>> {
         let devices = self.available_devices().await?;
-
-        if devices.is_empty() {
-            tracing::warn!("No device found. Please make sure you already setup Spotify Connect \
-                            support as described in https://github.com/aome510/spotify-player#spotify-connect.");
-        } else {
-            tracing::info!("Available devices: {devices:?}");
-        }
+        tracing::info!("Available devices: {devices:?}");
 
         // if there is an active device, return it
         if let Some(d) = devices.iter().find(|d| d.is_active) {
@@ -781,7 +807,6 @@ impl Client {
             .http_get::<rspotify::model::Page<rspotify::model::SimplifiedPlaylist>>(
                 &format!("{SPOTIFY_API_ENDPOINT}/me/playlists"),
                 &Query::from([("limit", "50")]),
-                false,
             )
             .await?;
         // let first_page = self
@@ -810,7 +835,7 @@ impl Client {
         let mut maybe_next = first_page.next;
         while let Some(url) = maybe_next {
             let mut next_page = self
-                .http_get::<rspotify::model::CursorPageFullArtists>(&url, &Query::new(), false)
+                .http_get::<rspotify::model::CursorPageFullArtists>(&url, &Query::new())
                 .await?
                 .artists;
             artists.append(&mut next_page.items);
@@ -879,7 +904,7 @@ impl Client {
             .into_iter()
             .filter_map(Album::try_from_simplified_album)
             .collect();
-        Ok(Client::process_artist_albums(albums))
+        Ok(AppClient::process_artist_albums(albums))
     }
 
     /// Start a playback
@@ -1309,7 +1334,6 @@ impl Client {
             .http_get::<rspotify::model::FullPlaylist>(
                 &format!("{SPOTIFY_API_ENDPOINT}/playlists/{}", playlist_id.id()),
                 &market_query(),
-                false,
             )
             .await?;
 
@@ -1441,12 +1465,7 @@ impl Client {
     }
 
     /// Make a GET HTTP request to the Spotify server
-    async fn http_get<T>(
-        &self,
-        url: &str,
-        payload: &Query<'_>,
-        use_user_client_id: bool,
-    ) -> Result<T>
+    async fn http_get<T>(&self, url: &str, payload: &Query<'_>) -> Result<T>
     where
         T: serde::de::DeserializeOwned,
     {
@@ -1462,13 +1481,7 @@ impl Client {
                 .replace("\"name\":null", "\"name\":\"\"")
         }
 
-        let access_token = if use_user_client_id {
-            self.access_token_from_user_client_id().await
-        } else {
-            self.access_token().await
-        }
-        .context("get access token")?;
-
+        let access_token = self.access_token().await?;
         tracing::debug!("{access_token} {url}");
 
         let response = self
@@ -1507,7 +1520,7 @@ impl Client {
 
         while let Some(url) = maybe_next {
             let mut next_page = self
-                .http_get::<rspotify::model::Page<T>>(&url, payload, false)
+                .http_get::<rspotify::model::Page<T>>(&url, payload)
                 .await?;
             if next_page.items.is_empty() {
                 break;
@@ -1530,7 +1543,7 @@ impl Client {
         let mut maybe_next = first_page.next;
         while let Some(url) = maybe_next {
             let mut next_page = self
-                .http_get::<rspotify::model::CursorBasedPage<T>>(&url, &Query::new(), false)
+                .http_get::<rspotify::model::CursorBasedPage<T>>(&url, &Query::new())
                 .await?;
             items.append(&mut next_page.items);
             maybe_next = next_page.next;
@@ -1560,7 +1573,7 @@ impl Client {
             let prev_name = match prev_item {
                 Some(rspotify::model::PlayableItem::Track(track)) => track.name.clone(),
                 Some(rspotify::model::PlayableItem::Episode(episode)) => episode.name.clone(),
-                None => String::new(),
+                Some(rspotify::model::PlayableItem::Unknown(_)) | None => String::new(),
             };
 
             player.playback = playback;
@@ -1571,7 +1584,7 @@ impl Client {
             let curr_name = match curr_item {
                 Some(rspotify::model::PlayableItem::Track(track)) => track.name.clone(),
                 Some(rspotify::model::PlayableItem::Episode(episode)) => episode.name.clone(),
-                None => String::new(),
+                Some(rspotify::model::PlayableItem::Unknown(_)) | None => String::new(),
             };
 
             let new_playback = prev_name != curr_name && !curr_name.is_empty();
@@ -1622,6 +1635,39 @@ impl Client {
             track_or_episode.clone()
         };
 
+        // retrieve current artist for genres if not in cache
+        let curr_artist = match &curr_item {
+            rspotify::model::PlayableItem::Track(full_track) => {
+                let cached = state
+                    .data
+                    .read()
+                    .caches
+                    .genres
+                    .contains_key(&full_track.artists[0].name);
+
+                if cached {
+                    None
+                } else {
+                    match &full_track.artists[0].id {
+                        Some(id) => self.spotify.artist(id.clone()).await.ok(),
+                        None => None,
+                    }
+                }
+            }
+            rspotify::model::PlayableItem::Episode(_)
+            | rspotify::model::PlayableItem::Unknown(_) => None,
+        };
+
+        if let Some(artist) = curr_artist {
+            if !artist.genres.is_empty() {
+                state.data.write().caches.genres.insert(
+                    artist.name,
+                    artist.genres,
+                    *TTL_CACHE_DURATION,
+                );
+            }
+        }
+
         let url = match curr_item {
             rspotify::model::PlayableItem::Track(ref track) => {
                 crate::utils::get_track_album_image_url(track)
@@ -1631,6 +1677,7 @@ impl Client {
                 crate::utils::get_episode_show_image_url(episode)
                     .ok_or(anyhow::anyhow!("missing image"))?
             }
+            rspotify::model::PlayableItem::Unknown(_) => return Ok(()),
         };
 
         let filename = (match curr_item {
@@ -1652,6 +1699,7 @@ impl Client {
                     &episode.show.id.as_ref().id()[..6]
                 )
             }
+            rspotify::model::PlayableItem::Unknown(_) => return Ok(()),
         })
         .replace('/', ""); // remove invalid characters from the file's name
         let path = configs.cache_folder.join("image").join(filename);
@@ -1663,8 +1711,19 @@ impl Client {
         #[cfg(feature = "image")]
         if !state.data.read().caches.images.contains_key(url) {
             let bytes = self.retrieve_image(url, &path, false).await?;
+
+            #[cfg(not(feature = "pixelate"))]
             let image =
                 image::load_from_memory(&bytes).context("Failed to load image from memory")?;
+            #[cfg(feature = "pixelate")]
+            let mut image =
+                image::load_from_memory(&bytes).context("Failed to load image from memory")?;
+
+            #[cfg(feature = "pixelate")]
+            {
+                Self::pixelate_image(&mut image);
+            }
+
             state
                 .data
                 .write()
@@ -1752,6 +1811,7 @@ impl Client {
                         let name = match playable {
                             rspotify::model::PlayableItem::Track(ref track) => &track.name,
                             rspotify::model::PlayableItem::Episode(ref episode) => &episode.name,
+                            rspotify::model::PlayableItem::Unknown(_) => continue,
                         };
                         text += name;
                     }
@@ -1767,6 +1827,7 @@ impl Client {
                         rspotify::model::PlayableItem::Episode(ref episode) => {
                             text += &episode.show.name;
                         }
+                        rspotify::model::PlayableItem::Unknown(_) => {}
                     },
                     &_ => {}
                 }
@@ -1833,6 +1894,17 @@ impl Client {
         Ok(bytes.to_vec())
     }
 
+    #[cfg(feature = "pixelate")]
+    fn pixelate_image(image: &mut image::DynamicImage) {
+        let pixels = config::get_config().app_config.cover_img_pixels;
+        let pixelated_image = image.resize(pixels, pixels, image::imageops::FilterType::Nearest);
+        *image = pixelated_image.resize(
+            image.width(),
+            image.height(),
+            image::imageops::FilterType::Nearest,
+        );
+    }
+
     /// Process a list of albums, which includes
     /// - sort albums by the release date
     /// - sort albums by the type if `sort_artist_albums_by_type` config is enabled
