@@ -37,7 +37,7 @@ pub fn get_episode_show_image_url(episode: &rspotify::model::FullEpisode) -> Opt
     }
 }
 
-pub fn parse_uri(uri: &str) -> Cow<str> {
+pub fn parse_uri(uri: &str) -> Cow<'_, str> {
     let parts = uri.split(':').collect::<Vec<_>>();
     // The below URI probably has a format of `spotify:user:{user_id}:{type}:{id}`,
     // but `rspotify` library expects to receive an URI of format `spotify:{type}:{id}`.
@@ -49,3 +49,30 @@ pub fn parse_uri(uri: &str) -> Cow<str> {
         Cow::Borrowed(uri)
     }
 }
+
+/// Get a list of items filtered by a search query.
+pub fn filtered_items_from_query<'a, T: std::fmt::Display>(
+    query: &str,
+    items: &'a [T],
+) -> Vec<&'a T> {
+    let query = query.to_lowercase();
+
+    #[cfg(feature = "fzf")]
+    return fuzzy_search_items(items, &query);
+
+    #[cfg(not(feature = "fzf"))]
+    items
+        .iter()
+        .filter(|t| {
+            if query.is_empty() {
+                true
+            } else {
+                let t = t.to_string().to_lowercase();
+                query
+                    .split(' ')
+                    .filter(|q| !q.is_empty())
+                    .all(|q| t.contains(q))
+            }
+        })
+        .collect::<Vec<_>>()
+}
